/* global console, define, window, $ */
define([], function () {

    "use strict";

    var core = {};

    console.log("init core");

    // =========================================================================
    // helper: create object with prototype
    // =========================================================================
    core.object = function (prototype) {
        var TmpFunction = function () {};
        TmpFunction.prototype = prototype;
        return new TmpFunction();
    };

    // =========================================================================
    // helper: extend object
    // =========================================================================
    core.extend = function (o1, o2) {
        for (var key in o2) {
            o1[key] = o2[key];
        }
        return o1;
    };

    // =========================================================================
    // bind
    // =========================================================================
    core.bind = function(obj,method){
        return function(){
            return method.apply(obj,arguments);
        };
    };

    // =========================================================================
    // helper: generate constructor function
    // =========================================================================
    var generateConstructorFunction = function () {
        var ConstructorFunction = null;
        ConstructorFunction = function () {
            if (!(this instanceof ConstructorFunction)) {
                return new ConstructorFunction("blub", arguments);
            }
            if (this.init) {
                if (arguments.length == 2 && arguments[0] === "blub") {
                    this.init.apply(this, arguments[1]);
                } else {
                    this.init.apply(this, arguments);
                }
            }
        };
        return ConstructorFunction;
    };

    // =========================================================================
    // add default methods
    // =========================================================================
    core.addDefaultMethods = function (prototype) {
        if (prototype.toString() === '[object Object]') {
            prototype.toString = function () {
                return "core.object";
            };
        }
    };

    // =========================================================================
    // create class
    // =========================================================================
    core.createClass = function (prototype) {
        var Cls = generateConstructorFunction();
        Cls.prototype = prototype;
        Cls.prototype.constructor = Cls;
        core.addDefaultMethods(Cls.prototype);
        return Cls;
    };

    // =========================================================================
    // create derived class
    // =========================================================================
    core.createDerivedClass = function (parentClass, prototype) {
        var Cls = generateConstructorFunction();
        Cls.prototype = core.extend(core.object(parentClass.prototype), prototype);
        Cls.prototype.constructor = Cls;
        core.addDefaultMethods(Cls.prototype);
        return Cls;
    };

    // =========================================================================
    // url manager
    // =========================================================================
    core.url = core.createClass({

        init: function () {
            this.load();
        },

        load: function () {
            this.parameters = {};
            var parameterString = window.location.search.substring(1);
            var parameters = parameterString.split("&");
            for (var i = 0; i < parameters.length; i++) {
                if (parameters[i].length === 0) {
                    continue;
                }
                var pair = parameters[i].split("=");
                var name = this.decode(pair[0]);
                var value = this.decode(pair[1]);
                this.parameters[name] = value;
            }
        },

        parameter: function (name, value) {
            if (arguments.length === 1) {
                return this.parameters[name];
            } else {
                this.parameters[name] = value;
            }
            return this;
        },

        parameterJSON: function (name, value) {
            if (arguments.length === 1) {
                var value2 = this.parameters[name];
                if (typeof value2 === 'undefined') {
                    return value2;
                }
                return JSON.parse(value2);
            } else {
                this.parameters[name] = JSON.stringify(value);
            }
            return this;
        },

        submit: function () {
            var parameterString = "";
            var first = true;
            for (var parameterName in this.parameters) {
                if (first) {
                    first = false;
                } else {
                    parameterString += "&";
                }
                var nameString = this.encode(parameterName);
                var valueString = this.encode(this.parameters[parameterName]);
                parameterString += nameString + "=" + valueString;
            }
            var newUrl = window.location.protocol + "//" + window.location.host + window.location.pathname;
            newUrl += "?" + parameterString;
            window.history.pushState("dummy", "Title", newUrl);
        },

        encode: function (text) {
            return encodeURIComponent(text);
        },

        decode: function (text) {
            return decodeURIComponent(text.replace(/\+/g, " "));
        }

    });

    // =========================================================================
    // table generator
    // =========================================================================
    core.Table = core.createClass({

        init: function () {
            this.rows = [];
        },

        addRow: function (row) {
            this.rows.push(row);
        },

        mirror: function () {

            var newRows = [];
            var lRows = this.rows.length;
            var lCols = this.rows[0].length;

            for (var c = 0; c < lCols; ++c) {
                var newRow = [];
                newRows.push(newRow);
                for (var r = 0; r < lRows; ++r) {
                    newRow.push(this.rows[r][c]);
                }
            }

            this.rows = newRows;
        },

        render: function () {

            var tableNode = $("<table></table>");

            for (var i = 0; i < this.rows.length; ++i) {
                var row = this.rows[i];
                var rowNode = $("<tr></tr>");
                tableNode.append(rowNode);
                for (var j = 0; j < row.length; ++j) {
                    var data = row[j];
                    var dataNode = $("<td></td>");
                    dataNode.append(data);
                    rowNode.append(dataNode);
                }
            }

            return tableNode;
        }

    });

    // =========================================================================
    // print
    // =========================================================================
    core.Printer = core.createClass({
        
        heading : '======================================================================',
        
        init: function (parentNode) {
            this.logNode = $("<pre></pre>");
            $(parentNode).append(this.logNode);
        },

        print: function () {
            console.log.apply(console,arguments);
            for (var i = 0; i < arguments.length; ++i) {
                this.logNode.append(arguments[i]);
            }
            this.logNode.append("\n");
        },
        
        printHeading : function(){
            this.print();
            this.print.apply(this,arguments);
            this.print(this.heading.slice(0,arguments[0].length));
            this.print();
        },

    });

    return core;

});