(function(global) {

	// =========================================================================
	// package creation & package aliases
	// =========================================================================

	global.astro.properties = {};

	var properties = global.astro.properties;
	var core = global.astro.core;
	var $ = global.$;

	// =========================================================================
	// property editor
	// =========================================================================

	properties.editor = core.createClass({

		init : function(parentSelector, model) {

			var self = this;
			self.parent = $(parentSelector);
			self.model = model;
			self.container = $("<div class='properties'></div>");
			self.parent.append(self.container);
			self.init2();
			self.model.refreshPropertyEditor = function() {
				self.init2();
			};
		},

		init2 : function() {

			var self = this;
			self.container.empty();

			var table = $("<table></table>");
			self.container.append(table);
			self.fieldMap = {};
			$.each(self.model.getProperties(), function(index, property) {

				// getter
				var getter = function() {
					var value = null;
					if (property.get) {
						value = property.get.apply(self.model);
					} else {
						if (property.object) {
							value = property.object[property.name];
						} else {
							value = self.model[property.name];
						}
					}
					if (property.convertToUI) {
						value = property.convertToUI(value);
					}
					return value;
				};

				// setter
				var setter = function(value) {
					if (property.convertFromUI) {
						value = property.convertFromUI(value);
					}
					if (property.set) {
						property.set.apply(self.model, [value]);
					} else {
						if (property.object) {
							property.object[property.name] = value;
						} else {
							self.model[property.name] = value;
						}

					}
				};

				// new table row
				var tr = $("<tr></tr>");
				table.append(tr);

				// add label column
				var labelField = $("<td></td>");
				tr.append(labelField);
				labelField.text(property.label);

				// add value column
				var valueField = $("<td></td>");
				tr.append(valueField);
				var inputField = null;
				if (property.type) {
					switch(property.type) {
						case 'dropdown':
							inputField = $("<select></select>");
							for (var i = 0; i < property.values.length; ++i) {
								var value = property.values[i];
								if (value === getter()) {
									inputField.append($("<option value='" + value + "' selected='selected'>" + value + "</option>"));
								} else {
									inputField.append($("<option value='" + value + "'>" + value + "</option>"));
								}
							}
							break;
						case 'dummy':
							break;
					}
				} else {
					inputField = $("<input></input>");
				}
				valueField.append(inputField);

				// disable setter for readonly
				if (property.readonly) {
					inputField.attr("readonly", "readonly");
					setter = function(value) {
					};
				}

				// set value
				inputField.attr("value", getter());

				// immediate input
				if (property.immediateInput) {
					inputField.change(function() {
						setter(inputField.attr("value"));
					});
				}

				// store field in map
				self.fieldMap[property.label] = {
					set : setter,
					get : getter,
					inputField : inputField
				};

			});

			// message area
			self.messageField = $("<div class='messagearea'></div>");
			self.container.append(self.messageField);

			// ok button
			var okButton = $("<button>OK</button>");
			self.container.append(okButton);
			okButton.click(function() {
				self.ok();
			});
		},

		ok : function() {
			var self = this;

			// reset background color + message area
			self.messageField.text("");
			$.each(self.fieldMap, function(label, fieldProperties) {
				fieldProperties.inputField.removeClass("error");
			});

			// assemble cleanup function
			var changed = false;
			var cleanup = function() {
				if (changed) {
					if (self.model.endOfUpdate) {
						self.model.endOfUpdate();
					}
					self.updateFields();
				}
			};

			// evaluate input
			$.each(self.fieldMap, function(label, fieldProperties) {
				try {
					var value = fieldProperties.inputField.attr("value");
					if (value != fieldProperties.get()) {
						fieldProperties.set(value);
						changed = true;
					}
				} catch(e) {
					fieldProperties.inputField.addClass("error");
					self.messageField.text(e);
					cleanup();
					return;
				}
			});

			cleanup();

		},

		updateFields : function() {
			var self = this;
			$.each(self.fieldMap, function(label, fieldProperties) {
				var value = fieldProperties.get();
				fieldProperties.inputField.attr("value", value);
			});
		}
	});

	// =========================================================================
	// test
	// =========================================================================

	properties.testEditor = function() {

		var testmodel = core.createClass({

			init : function() {
				this.a = 1;
				this.b = 2;
			},

			getProperties : function() {
				return [{
					label : 'a',
					name : 'a'
				}, {
					label : 'b',
					set : this.setB,
					get : this.getB
				}];
			},

			getB : function() {
				return this.b;
			},

			setB : function(b) {
				var value = parseInt(b, 10);
				if (isNaN(value)) {
					throw "not a number";
				}
				this.b = value;
			}
		});

		var m = testmodel();
		properties.editor($("body"), m);

	};

})(this);
