/* global console, define, window, $, document, MathJax */
define(["./astro.core", "./astro.math", "./astro.calc", "./astro.map", "./astro.data", "jquery"], function (core, math, calc, map, data, $) {

    var shorttheory = {};
    var module = shorttheory;

    module.drawSolarSystem = function (planets, paper) {

        var range = Math.max.apply(null, planets.map(function (planet) {
            return planet.getDistance();
        })) * 1.1;

        var mapObj = map.polarMap({
            paper: paper,
            realWidth: 2 * range,
            realHeight: 2 * range
        });

        var earth = null;
        var planet;
        for (var i = 0; i < planets.length; ++i) {
            planet = planets[i];
            mapObj.drawPlanet(planet);
            if (planet.title === 'Earth') {
                earth = planet;
            }
        }

        var scaleRad = earth.getDistance() / 1.5;
        mapObj.drawScale(earth, scaleRad);

        for (i = 0; i < planets.length; ++i) {
            planet = planets[i];
            var d = planet.subtract(earth);
            var dn = d.normalize();
            var length = Math.max(d.getDistance(), scaleRad) * 1.1;
            mapObj.drawLine(earth, planet.add(dn.multiply(length)));
        }

    };

    module.drawSinglePlanetSolarSystem = function (v, paper) {

        var range = v.getDistance() * 1.1;
        var mapObj = map.polarMap({
            paper: paper,
            realWidth: 2 * range,
            realHeight: 2 * range
        });
        mapObj.drawPlanet(v);

    };

    module.createMeanLengthCalculator = function (parent, planetId) {

        var table = $("<table></table>");
        parent.append(table);

        // === row: time
        var row = $("<tr></tr>");
        table.append(row);

        // cell: time
        row.append("<td>\\( T \\)</td>");

        // cell: time input
        var cell = $("<td></td>");
        var timeInput = $("<input id='sh_ml_time_" + planetId + "' type='text'>");
        cell.append(timeInput);
        row.append(cell);

        // cell: time dimension
        row.append($("<td>Jhd. seit 1.1.2000</td>"));

        // cell: copy button
        cell = $("<td></td>");
        row.append(cell);

        // === row: radius
        row = $("<tr></tr>");
        table.append(row);

        // cell: r
        row.append("<td>\\( r \\)</td>");

        // cell: r input
        cell = $("<td></td>");
        row.append(cell);
        var radiusInput = $("<input type='text' id='sh_ml_radius_" + planetId + "'>");
        cell.append(radiusInput);

        // cell: r dimension
        row.append($("<td>AE</td>"));

        // cell: planet dropdown
        cell = $("<td></td>");
        var planetDropdown = $("<select></select>");
        var planets = {};
        $.each(data, function () {
            var opt = $("<option>" + this.name + "</option>");
            if (this.id === planetId) {
                opt.attr("selected", "selected");
            }
            planetDropdown.append(opt);
            planets[this.name] = this;
        });
        var copy = function () {
            radiusInput.val(planets[this.value].semiMajorAxis[0]);
            meanLongitude0Input.val(planets[this.value].meanLongitude[0]);
            meanLongitudeDeltaInput.val(planets[this.value].meanLongitude[1]);
        };
        planetDropdown.change(copy);
        cell.append(planetDropdown);
        row.append(cell);

        // === row: mean longitude 0
        row = $("<tr></tr>");
        table.append(row);

        // cell: mean longitude
        row.append("<td>\\( L_0 \\)</td>");

        // cell: mean longitude input
        cell = $("<td></td>");
        var meanLongitude0Input = $("<input type='text'>");
        cell.append(meanLongitude0Input);
        row.append(cell);

        // cell: mean longitude dimension
        row.append($("<td>Grad</td>"));

        // === row: delta mean longitude
        row = $("<tr></tr>");
        table.append(row);

        // cell: delta mean longitude
        row.append("<td>\\( \\dot{L}  \\)</td>");

        // cell: delta mean longitude input
        cell = $("<td></td>");
        var meanLongitudeDeltaInput = $("<input type='text'>");
        cell.append(meanLongitudeDeltaInput);
        row.append(cell);

        // cell: delta mean longitude dimension
        row.append($("<td>Grad/Jahrhundert</td>"));

        // === row: calculate button
        row = $("<tr></tr>");
        table.append(row);

        // cell: calculate button
        cell = $("<td></td>");
        var calculateButton = $("<button>Berechnen</button>");
        cell.append(calculateButton);
        row.append(cell);
        calculateButton.click(function () {
            var l = parseFloat(meanLongitude0Input.val()) + parseFloat(meanLongitudeDeltaInput.val()) * parseFloat(timeInput.val());
            var l2 = math.rad2Deg(math.limit02Pi(math.deg2Rad(l)));
            meanLongitudeInput.val(l);
            meanLongitudeRInput.val(l2);
        });

        // === row: mean longitude
        row = $("<tr></tr>");
        table.append(row);

        // cell: mean longitude
        row.append("<td>\\( L = L_0 + \\dot{L} T \\)</td>");

        // cell: mean longitude input
        cell = $("<td></td>");
        var meanLongitudeInput = $("<input type='text' id='sh_ml_meanlongitude_" + planetId + "' readonly>");
        cell.append(meanLongitudeInput);
        row.append(cell);

        // cell: mean longitude dimension
        row.append($("<td>Grad</td>"));

        // === row: mean longitude reduced 0..360
        row = $("<tr></tr>");
        table.append(row);

        // cell: mean longitude reduced
        row.append("<td>\\( L \\) reduziert auf Bereich 0..360Grad</td>");

        // cell: mean longitude reduced input
        cell = $("<td></td>");
        var meanLongitudeRInput = $("<input type='text' readonly>");
        cell.append(meanLongitudeRInput);
        row.append(cell);

        // cell: mean longitude reduced dimension
        row.append($("<td>Grad</td>"));

        // fill with initial values
        copy.apply({
            value: data[planetId].name
        });
        
        // call mathjax
        MathJax.Hub.Queue(["Typeset",MathJax.Hub,table.get(0)]);

    };

    return module;
});